@@ -6,7 +6,7 @@ Created on Thu Jul  9 11:39:58 2020
 """
 # Import UI
 from PySide2 import QtCore, QtGui, QtWidgets
-from PySide2.QtWebEngineWidgets import QWebEngineView, QWebEngineDownloadItem, QWebEngineProfile
+from PySide2.QtWebEngineWidgets import QWebEngineView, QWebEngineProfile
 from .Table_from_pandas import *
 from . import PFAS_SAT_ui
 from . import MC_ui
@@ -21,25 +21,25 @@ import os
 import io
 import csv
 import sys
-import ast
-import pickle
 import importlib  #to import moduls with string name
 
 # Import General
 import pandas as pd
 import numpy as np
-from time import time
 from copy import deepcopy
 from pathlib import Path
 
 
 # Import PFAS_SAT
-from .. IncomFlow import IncomFlow
-from .. CommonData import CommonData
-from .. Inventory import Inventory
+from PFAS_SAT_ProcessModels import IncomFlow
+from PFAS_SAT_InputData import CommonData
+from PFAS_SAT_ProcessModels import Inventory
 from .. Project import Project
-from .. ProcessModelsMetaData import ProcessModelsMetaData
+from .. MCResults import MCResults
+from PFAS_SAT_ProcessModels import ProcessModelsMetaData
 import PFAS_SAT
+import PFAS_SAT_InputData
+import PFAS_SAT_ProcessModels
 
 # Import matplotlib
 from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
@@ -54,7 +54,7 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
         self.setupUi(self)
         self.init_app()
         
-        self.setWindowTitle('PFAS SAT (V {})'.format(PFAS_SAT.__version__))
+        self.setWindowTitle(f'PFAS SAT {PFAS_SAT.__version__} (InputData {PFAS_SAT_InputData.__version__}, ProcessModels {PFAS_SAT_ProcessModels.__version__})')
         
 
         self.WM_tab.setDisabled(True)
@@ -88,6 +88,8 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
         ### set setting
         self.CutOff = 0.001  # Fraction of total incoming PFAS
         self.AcceptableError = 5  # %
+        self.Included_PFAS = deepcopy(CommonData.PFAS_Index)
+        self._All_PFAS = deepcopy(CommonData.PFAS_Index)
         
 
     def init_app(self):
@@ -113,7 +115,8 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
         if not self.WM_tab_init_status:
             self.WM_tab.setEnabled(True)
             self.WM_DefData.setChecked(True)
-            self.IncomFlow = IncomFlow()
+            self.CommonData = CommonData()
+            self.IncomFlow = IncomFlow(CommonDataObject=self.CommonData)
             self.WM_Combo.clear()
             self.WM_Combo.currentTextChanged.connect(self.Load_Waste_Prop_func)
             self.WM_Combo.addItems(sorted(self.IncomFlow.WasteMaterialsFullName))
@@ -162,14 +165,14 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
             input_data_path = self.WM_DataPath.text()
         else:
             input_data_path = None
-        self.IncomFlow = IncomFlow(input_data_path = input_data_path)
+        self.IncomFlow = IncomFlow(input_data_path=input_data_path, CommonDataObject=self.CommonData)
         self.WM_Combo.clear()
         self.WM_Combo.addItems(sorted(self.IncomFlow.WasteMaterialsFullName))
             
     @QtCore.Slot(str)
     def Load_Waste_Prop_func(self,waste):
         data = self.IncomFlow.InputData.Data[:][(self.IncomFlow.InputData.Data['Category']== waste) | (self.IncomFlow.InputData.Data['Category']==waste+' PFAS Concentration')]
-        Waste_Prop_model = Table_from_pandas_editable(data)
+        Waste_Prop_model = Table_from_pandas_editable(data, pop_up=self.msg_popup)
         self.WM_table_prop.setModel(Waste_Prop_model)
         self.WM_table_prop.installEventFilter(self)
         self.WM_table_prop.setSortingEnabled(True)
@@ -218,7 +221,7 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
             DF=self.IncomFlow.InputData.Data
             file_name = str(fileDialog.getSaveFileName(filter="CSV (*.csv)")[0])
             if file_name:
-                DF.to_csv(file_name)
+                DF.to_csv(file_name, index=False)
         return(helper)       
         
 
@@ -372,15 +375,14 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
             
     @QtCore.Slot()
     def import_PM_init_PMInput(self):
-        self.CommonData = CommonData()
-        self.Inventory = Inventory()
+        self.Inventory = Inventory(CommonDataObject=self.CommonData)
         for proc in self.AllProcessModels:
             clas_name= proc
             clas_file=  ProcessModelsMetaData[proc]['File'].split('.')[0]
-            module = importlib.import_module('PFAS_SAT.'+clas_file)
+            module = importlib.import_module('PFAS_SAT_ProcessModels.'+clas_file)
             model = module.__getattribute__(clas_name)
             self.AllProcessModels[proc]['Model'] = model(input_data_path=self.AllProcessModels[proc]['InputDataPath'],
-                                                         CommonDataObjct=self.CommonData,
+                                                         CommonDataObject=self.CommonData,
                                                          InventoryObject=self.Inventory,
                                                          Name=self.AllProcessModels[proc]['Name'])
         
@@ -424,7 +426,7 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
         Process = self.ProcessNamehelper[self.PM_Input_model.currentText()]
         Data = self.AllProcessModels[Process]['Model'].InputData.Data        
     
-        model = Table_from_pandas_editable(Data)
+        model = Table_from_pandas_editable(Data, pop_up=self.msg_popup)
         self.PM_InputData_Table.setModel(model)
         self.PM_InputData_Table.setColumnHidden(1,True)
         self.filter_uncertinty_PM_Data(self.PM_Show_Uncertainty.isChecked())
@@ -473,7 +475,7 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
             DF=self.AllProcessModels[Process]['Model'].InputData.Data
             file_name = str(fileDialog.getSaveFileName(filter="CSV (*.csv)")[0])
             if file_name:
-                DF.to_csv(file_name)
+                DF.to_csv(file_name, index=False)
         return(helper)  
 
 #%% SYS tab          
@@ -507,9 +509,12 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
         
     def init_project(self):
         if self.project_init_status:
+            self.CommonData.PFAS_Index = deepcopy(self.Included_PFAS)
+            self.Inventory.__init__(CommonDataObject=self.CommonData)
             self.IncomFlow.set_flow(self.FU.currentText(),float(self.FU_amount.text()))
             self.Inventory._acceptableError = self.AcceptableError
-            self.Project = Project(Inventory=self.Inventory,CommonData=self.CommonData,ProcessModels=self.AllProcessModels, pop_up=self.msg_popup)
+            self.Project = Project(Inventory=self.Inventory, CommonData=self.CommonData,
+                                   ProcessModels=self.AllProcessModels, pop_up=self.msg_popup)
             Process_set=self.Project.get_process_set(self.IncomFlow.Inc_flow)
             
             # Potential down stream processes for each process
@@ -663,8 +668,19 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
         self.FlowParams_TreeView.setModel(None)
         self.project_init_status=True
         QtWidgets.QApplication.processEvents()
+        
+        # Clear MC
         self.MC_tab.setEnabled(False)
+        self.MC_PBr.setValue(0)
+        
+        # Clear FA
+        self.html_figur.setUrl(None)
+        self.Inventory_table.clearSpans()
         self.FA_tab.setEnabled(False)
+
+        # Clear SA
+        self.SA_tab.setEnabled(False)
+        self.SA_Clear_func()
             
     @QtCore.Slot()
     def set_process_set_func(self):
@@ -679,7 +695,7 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
         self.FlowParams = self.Project.get_flow_params()
         print(self.FlowParams)
         
-        model = TreeView(self.FlowParams)
+        model = TreeView(self.FlowParams, self.msg_popup)
         self.FlowParams_TreeView.setModel(model)
         self.FlowParams_TreeView.expandAll()
         self.FlowParams_TreeView.resizeColumnToContents(0)
@@ -721,7 +737,6 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
             # set the htm webEngine
             self.html_figur = QWebEngineView(parent=self.Sankey_groupBox)
             self.Sankey_layout.addWidget(self.html_figur)
-            
             self.FA_tab_init_status = True
         
         self.FA_tab.setEnabled(True)
@@ -746,8 +761,6 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
     def setup_sankey(self):
         self.html_figur.setUrl(QtCore.QUrl.fromLocalFile(os.getcwd()+'\\sankey.html'))
         
-
-
 #%% MC tab          
 # =============================================================================
 # =============================================================================
@@ -761,9 +774,7 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
             
             self.MC_N_runs.setMinimum(1)
             self.MC_N_runs.setMaximum(1000000)
-            
-            self.MC_PFAS.clear()
-            self.MC_PFAS.addItems(['All']+[i for i in self.CommonData.PFAS_Index])
+            self.MC_N_runs.setValue(1000)
             
             self.MC_Seed.setMinimum(0)
             self.MC_Seed.setMaximum(1000000)
@@ -789,6 +800,9 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
             self.MC_Uncertain_table.installEventFilter(self)
             self.MC_Uncertain_table.setSortingEnabled(True)           
         
+        self.MC_PFAS.clear()
+        self.MC_PFAS.addItems(['All']+[i for i in self.CommonData.PFAS_Index])
+        
         self.MC_tab.setEnabled(True)
         self.PFAS_SAT_1.setCurrentWidget(self.MC_tab)    
         self.MC_Model.clear()
@@ -804,7 +818,7 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
         else:
             self._uncertain_data = self.Project.ProcessModels[self.Project._ProcessNameRef[process]]['Model'].InputData.Data
             
-        model = Table_from_pandas_editable(self._uncertain_data)
+        model = Table_from_pandas_editable(self._uncertain_data, pop_up=self.msg_popup)
         self.MC_Uncertain_table.setModel(model)
         self.MC_Uncertain_table.setColumnHidden(1,True)
         self.MC_Uncertain_table.setWordWrap(True)
@@ -848,7 +862,7 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
             
             file_name = str(fileDialog.getSaveFileName(filter="CSV (*.csv)")[0])
             if file_name:
-                DF.to_csv(file_name)
+                DF.to_csv(file_name, index=False)
         return(helper)  
 
 
@@ -865,9 +879,9 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
             self._uncertain_data = self.Project.ProcessModels[self.Project._ProcessNameRef[process]]['Model'].InputData.Data
             
         if self.MC_uncertain_filter.isChecked():
-            model = Table_from_pandas_editable(self._uncertain_data[:][self._uncertain_data['uncertainty_type']>1])
+            model = Table_from_pandas_editable(self._uncertain_data[:][self._uncertain_data['uncertainty_type']>1], pop_up=self.msg_popup)
         else:
-            model = Table_from_pandas_editable(self._uncertain_data)
+            model = Table_from_pandas_editable(self._uncertain_data, pop_up=self.msg_popup)
         self.MC_Uncertain_table.setModel(model)
         
         self.MC_Uncertain_table.setWordWrap(True)
@@ -880,15 +894,7 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
     @QtCore.Slot()
     def MC_Run_func(self):
         self.MC_is_runnning = True
-        #start = time()
         print(" \n Monte Carlo simulation is started. \n Number of iterations: {} \n".format(int(self.MC_N_runs.text())))
-        #self.Project.setup_MC(self.IncomFlow)
-        #MC_results_raw=self.Project.MC_Run(int(self.MC_N_runs.text()))
-        #self.MC_results  =self.Project.MC_Result_DF(MC_results_raw)
-        #total_time = round(time()-start)
-        #self.msg_popup('Monte Carlo Simulation','The monte carlo simulation is done successfully in {} seconds.'.format(total_time),'Information')
-        #print('The monte carlo simulation is done successfully in {} seconds. \n'.format(total_time))
-        
         MC_worker = Worker_MC(parent=self.MC_run,
                               project=self.Project,
                               InputFlow_object=self.IncomFlow,
@@ -917,7 +923,7 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
         def helper():
             file_name = str(fileDialog.getSaveFileName(filter="CSV (*.csv)")[0])
             if file_name:
-                self.MC_results.to_csv(file_name)
+                self.MC_results.results_df.to_csv(file_name, index=False)
         return(helper)        
         
 
@@ -935,7 +941,7 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
         self.MC_Widget.tabWidget.setCurrentWidget(self.MC_Widget.MC_Data)
         
         ### Data Tab
-        MC_res_table_model = Table_from_pandas(self.MC_results)
+        MC_res_table_model = Table_from_pandas(self.MC_results.results_df)
         self.MC_Widget.MC_Res_Table.setModel(MC_res_table_model)
         self.MC_Widget.MC_Res_Table.resizeColumnsToContents()
         self.MC_Widget.MC_Res_Table.installEventFilter(self)
@@ -960,10 +966,10 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
         lay2.addWidget(self.canvas_plot_dist_mc)
         #self.ax_plot_dist_mc = self.fig_plot_dist_mc.add_subplot(111)
         
-        self.MC_Widget.y_axis.addItems([str(x) for x in self.MC_results.columns])
-        self.MC_Widget.x_axis.addItems([str(x) for x in self.MC_results.columns])
+        self.MC_Widget.y_axis.addItems([str(x) for x in Inventory.REPORT_INDEX])
+        self.MC_Widget.x_axis.addItems([str(x) for x in self.MC_results.results_df.columns])
         self.MC_Widget.scatter.setChecked(True)
-        self.MC_Widget.param.addItems([str(x) for x in self.MC_results.columns])
+        self.MC_Widget.param.addItems([str(x) for x in self.MC_results.results_df.columns])
         self.MC_Widget.box.setChecked(True)
         
         
@@ -971,13 +977,8 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
         self.MC_Widget.Update_plot.clicked.connect(self.mc_plot_func)
         self.MC_Widget.Update_dist_fig.clicked.connect(self.mc_plot_dist_func)
         
-        ### Correlation tab
-        self.corr_data = self.MC_results[self.MC_results.columns[1:]].corr(method='pearson')
-        
-        self._MC_results_index = ['Water (10e-6g)','Soil (10e-6g)','Air (10e-6g)','Destroyed (10e-6g)','Stored (10e-6g)', 'Injection Well(10e-6g)','Reactivated GAC (10e-6g)']
-
         # Correlation plot
-        self.MC_Widget.Corr_param.addItems(self._MC_results_index)
+        self.MC_Widget.Corr_param.addItems(Inventory.REPORT_INDEX)
         self.MC_Widget.Corr_btm.clicked.connect(self.corr_plot_func)
         
         #Figure initialization _ correlation plot
@@ -995,30 +996,9 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
     def corr_plot_func(self):
         self.fig_corr.clear()
         self.ax_plot_corr = self.fig_corr.add_subplot(111)
-    
-        #ploting the DataFrame
-        param=list(self.corr_data.columns)
-        for x in self._MC_results_index:
-            param.remove(x)
-
-        corr_data_plot = self.corr_data[self.MC_Widget.Corr_param.currentText()][param]
-        sorted_corr=corr_data_plot.abs().sort_values(ascending=False)
-
-        #xlim = round(max(sorted_corr) ,1) 
-        xlim = 1  # if xlim >1 else xlim
-
-        if len(sorted_corr.index) <= 5:
-            index = sorted_corr.index
-        else:
-            index = sorted_corr.index[0:5]
-        
-        self.ax_plot_corr=corr_data_plot[index].plot(kind='barh',
-                                                     ax=self.ax_plot_corr)
-        #set lables
-        self.ax_plot_corr.set_xlabel('Correlation with {}'.format(self.MC_Widget.Corr_param.currentText()), fontsize=18)
-        self.ax_plot_corr.set_xlim(-xlim, xlim)
-        self.ax_plot_corr.tick_params(axis='both', which='major', labelsize=18,rotation='auto')
-        self.ax_plot_corr.tick_params(axis='both', which='minor', labelsize=16,rotation='auto')
+            
+        self.MC_results.plot_corr(param=self.MC_Widget.Corr_param.currentText(),
+                                  ax=self.ax_plot_corr, **{'fontsize':18})
 
         #set margins
         self.canvas_corr.draw()
@@ -1028,19 +1008,23 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
     def mc_plot_func(self):
         self.fig_plot_mc.clear()
         self.ax_plot_mc = self.fig_plot_mc.add_subplot(111)
-    
-        #ploting the DataFrame        
-        self.ax_plot_mc=self.MC_results.plot(kind='scatter' if self.MC_Widget.scatter.isChecked() else 'hexbin',
-                                        x=self.MC_results.columns[self.MC_Widget.x_axis.currentIndex()],
-                                        y=self.MC_results.columns[self.MC_Widget.y_axis.currentIndex()],
-                                        ax=self.ax_plot_mc)
-        #set lables
-        self.ax_plot_mc.set_title(str(self.IncomFlow.Inc_flow.FlowType), fontsize=18)
-        self.ax_plot_mc.set_ylabel(self.MC_Widget.y_axis.currentText(), fontsize=18)
-        self.ax_plot_mc.set_xlabel(self.MC_Widget.x_axis.currentText(), fontsize=18)
-        self.ax_plot_mc.tick_params(axis='both', which='major', labelsize=18,rotation='auto')
-        self.ax_plot_mc.tick_params(axis='both', which='minor', labelsize=16,rotation='auto')
+        
+        try:
+            x=eval(self.MC_Widget.x_axis.currentText())
+        except Exception:
+            x=self.MC_Widget.x_axis.currentText()
 
+        try:
+            y=eval(self.MC_Widget.y_axis.currentText())
+        except Exception:
+            y=self.MC_Widget.y_axis.currentText()
+    
+        self.MC_results.plot_data(x=x,
+                                  y=y,
+                                  fig_type='scatter' if self.MC_Widget.scatter.isChecked() else 'hexbin',
+                                  ax=self.ax_plot_mc,
+                                  **{'title': str(self.IncomFlow.Inc_flow.FlowType),
+                                     'fontsize': 18})
         #set margins
         self.canvas_plot_mc.draw()
         self.fig_plot_mc.set_tight_layout(True)
@@ -1057,23 +1041,16 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
         else:
             kind = 'density'
 
-        #ploting the DataFrame  
-        if kind == 'hist':
-            self.ax_plot_dist_mc=self.MC_results[self.MC_results.columns[self.MC_Widget.param.currentIndex()]].plot(kind=kind,
-                                        ax=self.ax_plot_dist_mc, bins=30)
-        else:
-            self.ax_plot_dist_mc=self.MC_results[self.MC_results.columns[self.MC_Widget.param.currentIndex()]].plot(kind=kind,
-                                        ax=self.ax_plot_dist_mc)
-            
-        
-
-        
-        #set lables
-        self.ax_plot_dist_mc.set_title(str(self.IncomFlow.Inc_flow.FlowType), fontsize=18)
-        plt.rcParams["font.size"] = "18"
-        self.ax_plot_dist_mc.tick_params(axis='both', which='major', labelsize=18,rotation='auto')
-        self.ax_plot_dist_mc.tick_params(axis='both', which='minor', labelsize=16,rotation='auto')
+        try:
+            param = eval(self.MC_Widget.param.currentText())
+        except Exception:
+            param = self.MC_Widget.param.currentText()
 
+        self.MC_results.plot_dist(param=param,
+                                  fig_type=kind,
+                                  ax=self.ax_plot_dist_mc,
+                                  **{'title': str(self.IncomFlow.Inc_flow.FlowType),
+                                     'fontsize': 18})
         #set margins
         self.canvas_plot_dist_mc.draw()
         self.fig_plot_dist_mc.set_tight_layout(True)
@@ -1087,9 +1064,6 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
     @QtCore.Slot()
     def init_SA_tab_func(self):
         if not self.SA_tab_init_status:
-            self.SA_tab.setEnabled(True)
-            self.SA_PFAS.clear()
-            self.SA_PFAS.addItems(['All']+[i for i in self.CommonData.PFAS_Index])
             self.SA_Model.currentTextChanged.connect(self.load_categories_func)
             self.SA_Category.currentTextChanged.connect(self.load_parameters_func)
             self.SA_Parameter.currentTextChanged.connect(self.load_paramInfo_func)
@@ -1097,6 +1071,14 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
             self.SA_Save.clicked.connect(self.SA_save_file())
             self.SA_Clear.clicked.connect(self.SA_Clear_func)
             self.SA_Plot.clicked.connect(self.show_sa_res_func)
+            self.SA_NStep.setMinimum(2)
+            self.SA_NStep.setMaximum(1000)
+            self.SA_NStep.setValue(25)
+            self.SA_tab_init_status = True
+        
+        self.SA_tab.setEnabled(True)
+        self.SA_PFAS.clear()
+        self.SA_PFAS.addItems(['All']+[i for i in self.CommonData.PFAS_Index])
 
         self.PFAS_SAT_1.setCurrentWidget(self.SA_tab)
         self.SA_Model.clear()
@@ -1146,7 +1128,16 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
 
                 if key == 'unit':
                     self.SA_Unit_1.setText(val)
-                    self.SA_Unit_2.setText(val)                    
+                    self.SA_Unit_2.setText(val)
+                elif key == 'minimum' and not pd.isna(val):
+                    self.SA_Start.setText(str(val))
+                elif key == 'minimum' and pd.isna(val):
+                    self.SA_Start.setText("")
+                elif key == 'maximum' and not pd.isna(val):
+                    self.SA_Stop.setText(str(val))
+                elif key == 'maximum' and pd.isna(val):
+                    self.SA_Stop.setText("")
+
         else:
             param = parameter.split(':')[0]
             param_dict = self.Project.ProcessModels[self.Project._ProcessNameRef[self.SA_Model.currentText()]]['Model'].InputData.Input_dict[self.SA_Category.currentText()][param]
@@ -1161,6 +1152,15 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
                 if key == 'unit':
                     self.SA_Unit_1.setText(val)
                     self.SA_Unit_2.setText(val)
+                elif key == 'minimum' and not pd.isna(val):
+                    self.SA_Start.setText(str(val))
+                elif key == 'minimum' and pd.isna(val):
+                    self.SA_Start.setText("")
+                elif key == 'maximum' and not pd.isna(val):
+                    self.SA_Stop.setText(str(val))
+                elif key == 'maximum' and pd.isna(val):
+                    self.SA_Stop.setText("")
+
                 
         self.SA_ParamInfo.setText(info)        
 
@@ -1177,12 +1177,13 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
                                                       Paramter=str.split(self.SA_Parameter.currentText(),':')[0],
                                                       Start=float(self.SA_Start.text()),
                                                       Stop=float(self.SA_Stop.text()),
-                                                      Nstep=int(self.SA_NStep.text()),
+                                                      Nstep=self.SA_NStep.value(),
                                                       TypeOfPFAS=self.SA_PFAS.currentText())
-        self.SA_Res = self.Project.Result_to_DF(SA_Res_raw)
+        self.SA_Res = MCResults(SA_Res_raw)
+        self.SA_Res.to_df()
     
         ### Data Tab
-        SA_Res_table_model = Table_from_pandas(self.SA_Res)
+        SA_Res_table_model = Table_from_pandas(self.SA_Res.results_df)
         self.SA_Results.setModel(SA_Res_table_model)
         self.SA_Results.resizeColumnsToContents()
         self.SA_Results.installEventFilter(self)
@@ -1195,7 +1196,7 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
         def helper():
             file_name = str(fileDialog.getSaveFileName(filter="CSV (*.csv)")[0])
             if file_name:
-                self.SA_Res.to_csv(file_name)
+                self.SA_Res.results_df.to_csv(file_name, index=False)
         return(helper) 
 
     @QtCore.Slot(str)
@@ -1231,8 +1232,8 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
         lay.addWidget(toolbar)
         lay.addWidget(self.canvas_plot_sa)
         
-        self.SA_Widget.y_axis.addItems([str(x) for x in self.SA_Res.columns])
-        self.SA_Widget.x_axis.addItems([str(x) for x in self.SA_Res.columns])
+        self.SA_Widget.y_axis.addItems([str(x) for x in self.SA_Res.results_df.columns])
+        self.SA_Widget.x_axis.addItems([str(x) for x in self.SA_Res.results_df.columns])
     
         ### Connect the push bottoms
         self.SA_Widget.Update_plot.clicked.connect(self.sa_plot_func)
@@ -1244,19 +1245,23 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
     def sa_plot_func(self):
         self.fig_plot_sa.clear()
         self.ax_plot_sa = self.fig_plot_sa.add_subplot(111)
-    
-        #ploting the DataFrame        
-        self.ax_plot_sa=self.SA_Res.plot(kind='scatter',
-                                        x=self.SA_Res.columns[self.SA_Widget.x_axis.currentIndex()],
-                                        y=self.SA_Res.columns[self.SA_Widget.y_axis.currentIndex()],
-                                        ax=self.ax_plot_sa)
-        #set lables
-        self.ax_plot_sa.set_title(str(self.IncomFlow.Inc_flow.FlowType), fontsize=18)
-        self.ax_plot_sa.set_ylabel(self.SA_Widget.y_axis.currentText(), fontsize=18)
-        self.ax_plot_sa.set_xlabel(self.SA_Widget.x_axis.currentText(), fontsize=18)
-        self.ax_plot_sa.tick_params(axis='both', which='major', labelsize=18,rotation='auto')
-        self.ax_plot_sa.tick_params(axis='both', which='minor', labelsize=16,rotation='auto')
 
+        try:
+            x = eval(self.SA_Widget.x_axis.currentText())
+        except Exception:
+            x = self.SA_Widget.x_axis.currentText()
+
+        try:
+            y = eval(self.SA_Widget.y_axis.currentText())
+        except Exception:
+            y = self.SA_Widget.y_axis.currentText()
+    
+        self.SA_Res.plot_data(x=x,
+                              y=y,
+                              fig_type='scatter',
+                              ax=self.ax_plot_sa,
+                              **{'title': str(self.IncomFlow.Inc_flow.FlowType),
+                                 'fontsize': 18})    
         #set margins
         self.canvas_plot_sa.draw()
         self.fig_plot_sa.set_tight_layout(True)
@@ -1267,7 +1272,7 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
     ### General Functions
 # =============================================================================
 # =============================================================================   
-    def msg_popup(self,Subject,Information,Type):
+    def msg_popup(self, Subject, Information, Type):
         msg = QtWidgets.QMessageBox()
         if Type =='Warning':
             msg.setIcon(msg.Icon.Warning)
@@ -1406,8 +1411,8 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
         self.References = Reference_ui.Ui_References()
         self.References.setupUi(Dialog)
         
-        self.RefDF = pd.read_csv(Path(__file__).parent.parent/'Data/References.csv')
-        references = Table_from_pandas_editable(self.RefDF)
+        self.RefDF = pd.read_csv(Path(__file__).parent.parent/'InputData/Data/References.csv')
+        references = Table_from_pandas(self.RefDF)
         self.References.TableRef.setModel(references)
         self.References.TableRef.installEventFilter(self)
         self.References.TableRef.setWordWrap(True)
@@ -1426,7 +1431,7 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
     @QtCore.Slot()
     def Ref_filter_func(self):
         kwrd = self.References.kwrd.text()
-        references = Table_from_pandas_editable(self.RefDF[[kwrd in str(x) for x in self.RefDF['Reference']]])
+        references = Table_from_pandas(self.RefDF[[kwrd in str(x) for x in self.RefDF['Reference']]])
         self.References.TableRef.setModel(references)
         self.References.TableRef.installEventFilter(self)
         self.References.TableRef.setWordWrap(True)
@@ -1441,7 +1446,7 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
             DF=self.RefDF
             file_name = str(fileDialog.getSaveFileName(filter="CSV (*.csv)")[0])
             if file_name:
-                DF.to_csv(file_name)
+                DF.to_csv(file_name, index=False)
         return(helper) 
         
 
@@ -1460,6 +1465,22 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
         self.Setting.Apply.clicked.connect(self.Setting_apply_func)
         self.Setting.Cancel.clicked.connect(self.Setting_cancel_func)
         
+        self.pfas_choices = {}
+        x=0
+        y=0
+        for PFAS in self._All_PFAS:
+            if x==2:
+                x=0
+                y+=1
+            self.pfas_choices[PFAS] = QtWidgets.QCheckBox(self.Setting.PFAS_box)
+            self.pfas_choices[PFAS].setText(PFAS)
+            if PFAS in self.Included_PFAS:
+                self.pfas_choices[PFAS].setChecked(True)
+            else:
+                self.pfas_choices[PFAS].setChecked(False)
+            self.Setting.PFAS_Layout.addWidget(self.pfas_choices[PFAS], y, x, 1, 1)
+            x+=1
+
         self.Setting_Dialog.show()
         self.Setting_Dialog.exec_()
 
@@ -1468,6 +1489,12 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
         self.CutOff = float(self.Setting.CutOff.text())
         self.AcceptableError = float(self.Setting.ErrorLimit.text())
         self.Setting_Dialog.close()
+        for PFAS, checkbox in self.pfas_choices.items():
+            if checkbox.isChecked() and PFAS not in self.Included_PFAS:
+                self.Included_PFAS.append(PFAS)
+            elif not checkbox.isChecked() and PFAS in self.Included_PFAS:
+                self.Included_PFAS.remove(PFAS)
+        print(self.Included_PFAS)
 
     @QtCore.Slot()
     def Setting_cancel_func(self):
