@@ -4,9 +4,8 @@ Created on Wed Jul 29 17:03:27 2020
 
 @author: msmsa
 """
-import pandas as pd
 import numpy as np
-from .SubProcesses import split
+from PFAS_SAT_ProcessModels.SubProcesses import split
 import graphviz
 import plotly.graph_objects as go
 from plotly.offline import plot
@@ -15,7 +14,7 @@ import warnings
 import time
 from .utils import NpEncoder
 import importlib  # to import moduls with string name
-from .ProcessModelsMetaData import ProcessModelsMetaData
+from PFAS_SAT_ProcessModels import ProcessModelsMetaData
 
 
 class Project():
@@ -39,22 +38,30 @@ class Project():
                 for flow in ProcessModelsMetaData[P]['InputType']:
                     self.ProcessModels[P]['InputType'].append(flow)
                 clas_file = ProcessModelsMetaData[P]['File'].split('.')[0]
-                module = importlib.import_module('PFAS_SAT.'+clas_file)
+                module = importlib.import_module('PFAS_SAT_ProcessModels.' + clas_file)
                 model = module.__getattribute__(P)
                 self.ProcessModels[P]['Model'] = model(input_data_path=None,
-                                                       CommonDataObjct=self.CommonData,
+                                                       CommonDataObject=self.CommonData,
                                                        InventoryObject=self.Inventory,
                                                        Name=self.ProcessModels[P]['Name'])
 
         self.Processes = list(self.ProcessModels.keys())
         self._ProcessNameRef = {}
         for key, val in self.ProcessModels.items():
-                self._ProcessNameRef[val['Name']] = key
+            self._ProcessNameRef[val['Name']] = key
 
         self.WasteTreatment = {}
         for i in self.WasteMaterials:
             self.WasteTreatment[i] = self._find_destination(i)
-        print('\n Treatment options for each waste material: {} \n\n'.format(self.WasteTreatment))
+        msg = "=========================================================\n"
+        msg += "        Treatment options for each waste material        \n"
+        msg += "=========================================================\n"
+
+        for k, v in self.WasteTreatment.items():
+            msg += f'\n{k}\n'
+            msg += f'{v}\n'
+        msg += "=========================================================\n"
+        print(msg)
 
         # Print warnings
         warnings.simplefilter('always', UserWarning)
@@ -97,7 +104,9 @@ class Project():
                                 for PPPP in self.WasteTreatment[Product3]:
                                     ProcessSet.add(PPPP)
                                     ProcessSetQua.add(PPPP)
-        return(ProcessSet, ProcessSetPrim, ProcessSetSec, ProcessSetTer, ProcessSetQua)
+        for i, j in enumerate((ProcessSetPrim, ProcessSetSec, ProcessSetTer, ProcessSetQua)):
+            print(f"Level {i} options: {j}\n")
+        return ProcessSet, ProcessSetPrim, ProcessSetSec, ProcessSetTer, ProcessSetQua
 
     def set_process_set(self, ProcessSet):
         self.ProcessSet = ProcessSet
@@ -136,14 +145,17 @@ class Project():
         if normalize:
             for key, val in self.FlowParams.items():
                 if len(val) > 0:
+                    sum_norm = 0
                     for key2 in val:
-                        val[key2] = np.round(1/len(val), 4)
+                        sum_norm += np.round(1 / len(val), 4)
+                        val[key2] = np.round(1 / len(val), 4)
+                    val[key2] += np.round((1 - sum_norm), 4)
         return(self.FlowParams)
 
     def set_flow_params(self, FlowParams):
         for key, value in FlowParams.items():
             if len(value) > 0:
-                if abs(1-sum(value.values())) > 0.01:
+                if abs(1 - sum(value.values())) > 0.0001:
                     raise ValueError('Sum of the fractions for {} is not 1'.format(key))
         self.FlowParams = FlowParams
 
@@ -219,22 +231,13 @@ class Project():
                     time.sleep(0.1)
 
         self.Reset_static_Data()
-        return(MC_results)
+        return MC_results
 
     def Reset_static_Data(self):
         self.InputFlow_object.InputData.reset_static_vals()
         for p in self.Processes:
             self.ProcessModels[p]['Model'].InputData.reset_static_vals()
 
-    def Result_to_DF(self, results):
-        DF = pd.DataFrame()
-        DF['Iteration'] = [results[j][0] for j in range(len(results))]
-        for i in results[0][2].keys():
-            DF[i] = [results[j][2][i] for j in range(len(results))]
-        for i in range(len(results[0][1])):
-            DF[results[0][1][i][0]] = [results[j][1][i][1] for j in range(len(results))]
-        return(DF)
-
     def setup_SA(self, InputFlow_object):
         self.InputFlow_object = InputFlow_object
 
@@ -275,31 +278,31 @@ class Project():
             return(SA_results)
 
     def plot_network(self, view=True, show_vals=True, all_flow=True):
-            """
-            To render the generated DOT source code, you also need to install `Graphviz <https://www.graphviz.org/download>`_.
+        """
+        To render the generated DOT source code, you also need to install `Graphviz <https://www.graphviz.org/download>`_.
 
-            ..note:: Make sure that the directory containing the dot executable is on your systems path.
+        ..note:: Make sure that the directory containing the dot executable is on your systems path.
 
-            """
-            # Set the color for starting waste material
-            self._NtwkrNodeColor[self.InputFlow.FlowType] = 'chartreuse2'
-            # Initialize PFAS treatment network
-            self.network = graphviz.Digraph(name='Network', filename='Network.gv', format='png', engine='dot')
-            self.network.graph_attr['rankdir'] = 'LR'
-            self.network.attr(dpi='300')
-            self.network.attr(size='5,5')
-            for x in self._NtwkrNode:
-                self.network.node(x, shape=self._NtwkrNodeShape[x], fillcolor=self._NtwkrNodeColor[x], style='filled', width='1.2')
+        """
+        # Set the color for starting waste material
+        self._NtwkrNodeColor[self.InputFlow.FlowType] = 'chartreuse2'
+        # Initialize PFAS treatment network
+        self.network = graphviz.Digraph(name='Network', filename='Network.gv', format='png', engine='dot')
+        self.network.graph_attr['rankdir'] = 'LR'
+        self.network.attr(dpi='300')
+        self.network.attr(size='5,5')
+        for x in self._NtwkrNode:
+            self.network.node(x, shape=self._NtwkrNodeShape[x], fillcolor=self._NtwkrNodeColor[x], style='filled', width='1.2')
 
-            for e in self._Ntwkedge:
-                self.network.edge(e[0], e[1], label=None, color='black')
-            try:
-                self.network.render('Network', view=view)
-            except Exception:
-                print("""
-                      To render the generated DOT source code, you also need to install Graphviz (`Graphviz <https://www.graphviz.org/download>`_).\n
-                      Make sure that the directory containing the dot executable is on your systems’ path.
-                      """)
+        for e in self._Ntwkedge:
+            self.network.edge(e[0], e[1], label=None, color='black')
+        try:
+            self.network.render('Network', view=view)
+        except Exception:
+            print("""
+                  To render the generated DOT source code, you also need to install Graphviz (`Graphviz <https://www.graphviz.org/download>`_).\n
+                  Make sure that the directory containing the dot executable is on your systems’ path.
+                  """)
 
     def plot_sankey(self, view=True, filename=None):
 
@@ -364,13 +367,16 @@ class Project():
             else:
                 color_link.append('rgba(255,127, 0, 0.8)')
 
-        print("""
-              # Sankey Mass flows
-              label = {}
-              source = {}
-              target = {}
-              label_link = {}
-              value = {}""".format(label, source, target, label_link, value))
+        msg = "=============================\n"
+        msg += "       Sankey Mass flows     \n"
+        msg += "=============================\n"
+        msg += f"label = {label}\n"
+        msg += f"source = {source}\n"
+        msg += f"target = {target}\n"
+        msg += f"label_link = {label_link}\n"
+        msg += f"value = {value}\n"
+        msg += "=============================\n"
+        print(msg)
 
         node = dict(pad=20,
                     thickness=20,
@@ -405,6 +411,5 @@ class Project():
         store_data['node'] = node
         store_data['link'] = link
 
-        print(store_data)
         with open('Sankey_Data.JSON', 'w') as outfile:
             json.dump(store_data, outfile, indent=4, cls=NpEncoder)
